import os

ROOT_PATH = os.path.split(os.path.abspath(__name__))[0]

DEBUG = True
SECRET_KEY = '{{ secret_key }}'
SQLALCHEMY_DATABASE_URI = 'sqlite:///{}'.format('{{ project_name }}.db')
SQLALCHEMY_TRACK_MODIFICATIONS = False

